package io.zipcoder;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import java.util.ArrayList;
import java.util.Map;

import static org.junit.Assert.*;

public class ItemParserTest {

    private String rawSingleItem = "naMe:Milk;price:3.23;type:Food;expiration:1/25/2016##";

    private String rawSingleItemIrregularSeperatorSample = "naMe:MiLK;price:3.23;type:Food^expiration:1/11/2016##";

    private String rawBrokenSingleItem = "naMe:Milk;price:3.23;type:Food;expiration:1/25/2016##";

    private String rawBrokenSingleItem1 = "naMe:Milk;price:3.23;type:;expiration:1/25/2016##";

    private String rawMultipleItems = "naMe:Milk;price:3.23;type:Food;expiration:1/25/2016##"
            + "naME:BreaD;price:1.23;type:Food;expiration:1/02/2016##"
            + "NAMe:BrEAD;price:1.23;type:Food;expiration:2/25/2016##";
    private ItemParser itemParser;

    @Before
    public void setUp() {
        itemParser = new ItemParser();
    }

    @Test
    public void parseRawDataIntoStringArrayTest() {
        Integer expectedArraySize = 3;
        ArrayList<String> items = itemParser.parseRawDataIntoStringArray(rawMultipleItems);
        Integer actualArraySize = items.size();
        assertEquals(expectedArraySize, actualArraySize);
    }

    @Test
    public void parseStringIntoItemTest() throws ItemParseException {
        Item expected = new Item("milk", 3.23, "food", "1/25/2016");
        Item actual = itemParser.parseStringIntoItem(rawSingleItem);
        assertEquals(expected.toString(), actual.toString());
    }

//    @Test(expected = ItemParseException.class)
//    public void parseBrokenStringIntoItemTest() throws ItemParseException{
//        itemParser.parseStringIntoItem(rawBrokenSingleItem);
//    }

    @Test(expected = ItemParseException.class)
    public void parseBrokenStringIntoItemTest1() throws ItemParseException {
        itemParser.parseStringIntoItem(rawBrokenSingleItem1);
    }

    @Test
    public void findKeyValuePairsInRawItemDataTest() {
        Integer expected = 4;
        Integer actual = itemParser.findKeyValuePairsInRawItemData(rawSingleItem).size();
        assertEquals(expected, actual);
    }

    @Test
    public void findKeyValuePairsInRawItemDataTestIrregular() {
        Integer expected = 4;
        Integer actual = itemParser.findKeyValuePairsInRawItemData(rawSingleItemIrregularSeperatorSample).size();
        assertEquals(expected, actual);
    }

    @Test
    public void readFromFile() {
        String expected = "meow meow";
        String actual = itemParser.readFromFile("src/main/resources/TestFile.txt");

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void parseRawDataIntoItemsTest() {
        int expected = 3;
        int actual = itemParser.parseRawDataIntoItems(rawMultipleItems).size();

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void findValue() throws ItemParseException {
        String expected = "milk";
        String actual = itemParser.findValue(itemParser.findKeyValuePairsInRawItemData(rawSingleItem).get(0));

        Double expected1 = 3.23;
        Double actual1 = Double.valueOf(itemParser.findValue(itemParser.findKeyValuePairsInRawItemData(rawSingleItem).get(1)));

        String expected2 = "food";
        String actual2 = itemParser.findValue(itemParser.findKeyValuePairsInRawItemData(rawSingleItem).get(2));

        String expected3 = "1/25/2016";
        String actual3 = itemParser.findValue(itemParser.findKeyValuePairsInRawItemData(rawSingleItem).get(3));

        Assert.assertEquals(expected, actual);
        Assert.assertEquals(expected1, actual1);
        Assert.assertEquals(expected2, actual2);
        Assert.assertEquals(expected3, actual3);
    }

    @Test
    public void countItem() {
        Map<String, Integer> itemMap = itemParser.itemCounts(itemParser.parseRawDataIntoItems(rawMultipleItems));

        int expected = 2;
        int actual = itemMap.get("bread");

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void convertToLowerCase() {
        String expected = "meow meow meow";
        String actual = itemParser.convertToLowerCase("Meow mEOW meOw");

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void convertZeroTo0() {
        String expected = "meow";
        String actual = itemParser.convertZeroTo0("me0w");

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void capitalizeFirst() {
        String expected = "Meow";
        String actual = itemParser.capitalizeFirst("meow");

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void stringToArrayList() {
        ArrayList<String> expected = new ArrayList<>();
        expected.add("a");
        expected.add("b");
        expected.add("c");
        ArrayList<String> actual = itemParser.stringToArrayList("abc");

        Assert.assertEquals(expected, actual);
    }
}
